==> tw-svr5-base-1.0.3-20240306.071659-2-sources.jar: com/elitesland/tw/tw5/base/util/SqlUtil.java <==
package com.elitesland.tw.tw5.base.util;


import com.elitescloud.boot.model.entity.BaseModel;
import com.elitescloud.boot.model.entity.QBaseModel;
import com.elitescloud.cloudt.common.base.param.OrderItem;
import com.elitescloud.cloudt.system.vo.SysUserDTO;
import com.elitesland.tw.tw5.base.common.TwCommonPayload;
import com.elitesland.tw.tw5.base.common.TwException;
import com.elitesland.tw.tw5.base.common.TwQueryParam;
import com.querydsl.core.types.Order;
import com.querydsl.core.types.OrderSpecifier;
import com.querydsl.core.types.Path;
import com.querydsl.core.types.dsl.EntityPathBase;
import com.querydsl.core.types.dsl.Expressions;
import com.querydsl.jpa.impl.JPAQuery;
import com.querydsl.jpa.impl.JPAUpdateClause;
import org.springframework.util.ObjectUtils;

import java.lang.reflect.Field;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * sql工具类
 *
 * @author zhangyongqiang
 */
public class SqlUtil {

    public static String toSqlLikeString(String value) {
        return "%" + value + "%";
    }

    private static List<OrderItem> orderse = null;

    public static List<OrderItem> getOrderse() {
        if (orderse == null) {
            orderse = new ArrayList<>();
            orderse.add(OrderItem.asc("sortIndex"));
            orderse.add(OrderItem.desc("createTime"));
        }
        return orderse;
    }

    /**
     * 自定义分页时的排序字段
     *
     * @param pathBase Q对象
     * @param orders   排序数据
     * @return 排序order
     */
    public static <T> OrderSpecifier<?>[] getSortedColumn(EntityPathBase<T> pathBase, List<OrderItem> orders) {
        if (orders == null || orders.size() == 0) {
            Path<String> fieldPath = Expressions.path(String.class, pathBase, "createTime");
            return new OrderSpecifier[]{new OrderSpecifier<>(Order.DESC, fieldPath)};
        } else {
            OrderSpecifier<String>[] orderSpecifiers = new OrderSpecifier[orders.size()];
            for (int i = 0; i < orders.size(); i++) {
                OrderItem orderItem = orders.get(i);
                Order order = orderItem.isAsc() ? Order.ASC : Order.DESC;
                Path<String> fieldPath = Expressions.path(String.class, pathBase, orderItem.getColumn());
                orderSpecifiers[i] = new OrderSpecifier<>(order, fieldPath);
            }
            return orderSpecifiers;
        }
    }

    /**
     * 分页排序工具类，动态处理分页字段。 当pathBase参数有多个对象时，且排序字段存在多个表时，以靠前的Q对象优先
     *
     * @param orders   排序数据
     * @param pathBase Q对象数组
     * @return 排序order
     */
    @SuppressWarnings("rawtypes,unchecked")
    public static  OrderSpecifier<?>[] getSortedColumn(List<OrderItem> orders, EntityPathBase ... pathBase) {
        if (orders == null || orders.size() == 0) {
            Path<String> fieldPath = Expressions.path(String.class, pathBase[0], "createTime");
            return new OrderSpecifier[]{new OrderSpecifier<>(Order.DESC, fieldPath)};
        } else {
            OrderSpecifier<String>[] orderSpecifiers = new OrderSpecifier[orders.size()];
            for (int i = 0; i < orders.size(); i++) {
                OrderItem orderItem = orders.get(i);
                Order order = orderItem.isAsc() ? Order.ASC : Order.DESC;
                EntityPathBase ownerEntity = getOwnerEntity(orderItem.getColumn(), pathBase);
                if(ownerEntity == null){
                    throw TwException.error("","不存在要排序的字段");
                }
                Path<String> fieldPath = Expressions.path(String.class, ownerEntity, orderItem.getColumn());
                orderSpecifiers[i] = new OrderSpecifier<>(order, fieldPath);
            }
            return orderSpecifiers;
        }
    }

    /**
     * 自动动态处理nullFields的修改
     *
     * @param payload   需要设置为null的payload对象
     * @param pathBase Q对象
     * @param update JPAUpdateClause 对象
     */
    @SuppressWarnings("rawtypes")
    public static  void handleNullFieldsUpdate(TwCommonPayload payload, EntityPathBase  pathBase, JPAUpdateClause update) {
        List<String> nullFields = payload.getNullFields();
        if (nullFields != null && nullFields.size() > 0) {
            for (String nullField : nullFields) {
                Path<String> nullFieldPath = Expressions.path(String.class, pathBase, nullField);
                update.setNull(nullFieldPath);
            }
        }
    }


    // 存储Q对象名称与相对应字段
    private static final Map<String,List<String>> fieldMap = new HashMap<>();
    /**
     * 获得字段属于哪个Q对象
     *
     * @param column 字段名称
     * @param pathBase 多个Q对象
     * @return Q对象
     */
    @SuppressWarnings("rawtypes")
    private static  EntityPathBase getOwnerEntity(String column, EntityPathBase ... pathBase ){
        EntityPathBase result = null;
        for (EntityPathBase path : pathBase) {
            String className = path.getClass().getName();
            List<String> stringFields = fieldMap.get(className);
            if (stringFields == null) {
                Field[] declaredFields = path.getClass().getDeclaredFields();
                List<String> stringList = new ArrayList<>(declaredFields.length);
                for (Field declaredField : declaredFields) {
                    if (Path.class.isAssignableFrom(declaredField.getType())) {
                        stringList.add(declaredField.getName());
                    }
                }
                stringFields = stringList;
                fieldMap.put(className, stringList);
            }
            // 当前Q对象是否包含字段
            if (stringFields.contains(column)) {
                result = path;
                break;
            }
        }
        return result;
    }

    /**
     * 常用基础查询条件拼装
     *
     * @param jpaQuery   jpaQuery对象
     * @param qBaseModel qBaseModel
     * @param query      查询条件
     * @param <T>        范型对象
     */
    public static <T> void handleCommonJpaQuery(JPAQuery<T> jpaQuery, QBaseModel qBaseModel, TwQueryParam query) {
        jpaQuery.where(qBaseModel.deleteFlag.eq(0));
        if (!ObjectUtils.isEmpty(query.getId())) {
            jpaQuery.where(qBaseModel.id.eq(query.getId()));
        }
        if (!ObjectUtils.isEmpty(query.getIds())) {
            jpaQuery.where(qBaseModel.id.in(query.getIds()));
        }
        if (!ObjectUtils.isEmpty(query.getCreateUserId())) {
            jpaQuery.where(qBaseModel.createUserId.eq(query.getCreateUserId()));
        }
        if (!ObjectUtils.isEmpty(query.getCreateTimeStart())) {
            jpaQuery.where(qBaseModel.createTime.goe(query.getCreateTimeStart()));
        }
        if (!ObjectUtils.isEmpty(query.getCreateTimeEnd())) {
            jpaQuery.where(qBaseModel.createTime.loe(query.getCreateTimeEnd()));
        }
        if (!ObjectUtils.isEmpty(query.getRemark())) {
            jpaQuery.where(qBaseModel.remark.like(SqlUtil.toSqlLikeString(query.getRemark())));
        }
    }

    /**
     * 创建单据的时候审计字段会被前端传的值影响，强制清空相关字段
     * @param entity entity 对象
     */
    public static void clearAuditFields(BaseModel entity){
        if(entity != null){
            entity.setCreator(null);
            entity.setCreateUserId(null);
            entity.setCreateTime(null);
            entity.setUpdater(null);
            entity.setModifyTime(null);
            entity.setModifyUserId(null);
            entity.setDeleteFlag(0);
        }
    }

    /**
     * 创建单据的时候审计字段会被前端传的值影响，强制清空相关字段
     * @param entities entities 对象
     */
    public static void clearAuditFields(List<? extends BaseModel> entities){
        if(entities != null && entities.size() > 0){
            entities.forEach(SqlUtil::clearAuditFields);
        }
    }

    /**
     * 必要的更新字段
     *
     * Deprecated: 架构已自动处理审计字段
     * @param update     更新语句
     * @param qBaseModel 更新对象
     */
    @Deprecated()
    public static void updateCommonJpaQuery(JPAUpdateClause update, QBaseModel qBaseModel) {
        SysUserDTO userDTO = GlobalUtil.getLoginUser();
        update.set(qBaseModel.modifyUserId, userDTO.getId());
        update.set(qBaseModel.updater, userDTO.getFirstName());
        update.set(qBaseModel.modifyTime, LocalDateTime.now());
    }
}

==> tw-svr5-base-3.1.2-20240122.062351-1-sources.jar: com/elitesland/tw/tw5/base/util/SqlUtil.java <==
package com.elitesland.tw.tw5.base.util;


import com.elitescloud.boot.model.entity.QBaseModel;
import com.elitescloud.cloudt.common.base.param.OrderItem;
import com.elitescloud.cloudt.system.vo.SysUserDTO;
import com.elitesland.tw.tw5.base.common.TwException;
import com.elitesland.tw.tw5.base.common.TwQueryParam;
import com.querydsl.core.types.Order;
import com.querydsl.core.types.OrderSpecifier;
import com.querydsl.core.types.Path;
import com.querydsl.core.types.dsl.EntityPathBase;
import com.querydsl.core.types.dsl.Expressions;
import com.querydsl.jpa.impl.JPAQuery;
import com.querydsl.jpa.impl.JPAUpdateClause;
import org.springframework.util.ObjectUtils;

import java.lang.reflect.Field;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * sql工具类
 *
 * @author zhangyongqiang
 */
public class SqlUtil {

    public static String toSqlLikeString(String value) {
        return "%" + value + "%";
    }

    private static List<OrderItem> orderse = null;

    public static List<OrderItem> getOrderse() {
        if (orderse == null) {
            orderse = new ArrayList<>();
            orderse.add(OrderItem.asc("sortIndex"));
            orderse.add(OrderItem.desc("createTime"));
        }
        return orderse;
    }

    /**
     * 自定义分页时的排序字段
     *
     * @param pathBase Q对象
     * @param orders   排序数据
     * @return 排序order
     */
    public static <T> OrderSpecifier<?>[] getSortedColumn(EntityPathBase<T> pathBase, List<OrderItem> orders) {
        if (orders == null || orders.size() == 0) {
            Path<String> fieldPath = Expressions.path(String.class, pathBase, "createTime");
            return new OrderSpecifier[]{new OrderSpecifier<>(Order.DESC, fieldPath)};
        } else {
            OrderSpecifier<String>[] orderSpecifiers = new OrderSpecifier[orders.size()];
            for (int i = 0; i < orders.size(); i++) {
                OrderItem orderItem = orders.get(i);
                Order order = orderItem.isAsc() ? Order.ASC : Order.DESC;
                Path<String> fieldPath = Expressions.path(String.class, pathBase, orderItem.getColumn());
                orderSpecifiers[i] = new OrderSpecifier<>(order, fieldPath);
            }
            return orderSpecifiers;
        }
    }

    /**
     * 分页排序工具类，动态处理分页字段。 当pathBase参数有多个对象时，且排序字段存在多个表时，以靠前的Q对象优先
     *
     * @param orders   排序数据
     * @param pathBase Q对象数组
     * @return 排序order
     */
    @SuppressWarnings("rawtypes,unchecked")
    public static  OrderSpecifier<?>[] getSortedColumn(List<OrderItem> orders, EntityPathBase ... pathBase) {
        if (orders == null || orders.size() == 0) {
            Path<String> fieldPath = Expressions.path(String.class, pathBase[0], "createTime");
            return new OrderSpecifier[]{new OrderSpecifier<>(Order.DESC, fieldPath)};
        } else {
            OrderSpecifier<String>[] orderSpecifiers = new OrderSpecifier[orders.size()];
            for (int i = 0; i < orders.size(); i++) {
                OrderItem orderItem = orders.get(i);
                Order order = orderItem.isAsc() ? Order.ASC : Order.DESC;
                EntityPathBase ownerEntity = getOwnerEntity(orderItem.getColumn(), pathBase);
                if(ownerEntity == null){
                    throw TwException.error("","不存在要排序的字段");
                }
                Path<String> fieldPath = Expressions.path(String.class, ownerEntity, orderItem.getColumn());
                orderSpecifiers[i] = new OrderSpecifier<>(order, fieldPath);
            }
            return orderSpecifiers;
        }
    }

    /**
     * 自动动态处理nullFields的修改
     *
     * @param nullFields   需要设置为null的字段
     * @param pathBase Q对象
     * @param update JPAUpdateClause 对象
     */
    @SuppressWarnings("rawtypes")
    public static  void handleNullFieldsUpdate(List<String> nullFields, EntityPathBase  pathBase, JPAUpdateClause update) {
        if (nullFields != null && nullFields.size() > 0) {
            for (String nullField : nullFields) {
                Path<String> nullFieldPath = Expressions.path(String.class, pathBase, nullField);
                update.setNull(nullFieldPath);
            }
        }
    }


    // 存储Q对象名称与相对应字段
    private static final Map<String,List<String>> fieldMap = new HashMap<>();
    /**
     * 获得字段属于哪个Q对象
     *
     * @param column 字段名称
     * @param pathBase 多个Q对象
     * @return Q对象
     */
    @SuppressWarnings("rawtypes")
    private static  EntityPathBase getOwnerEntity(String column, EntityPathBase ... pathBase ){
        EntityPathBase result = null;
        for (EntityPathBase path : pathBase) {
            String className = path.getClass().getName();
            List<String> stringFields = fieldMap.get(className);
            if (stringFields == null) {
                Field[] declaredFields = path.getClass().getDeclaredFields();
                List<String> stringList = new ArrayList<>(declaredFields.length);
                for (Field declaredField : declaredFields) {
                    if (Path.class.isAssignableFrom(declaredField.getType())) {
                        stringList.add(declaredField.getName());
                    }
                }
                stringFields = stringList;
                fieldMap.put(className, stringList);
            }
            // 当前Q对象是否包含字段
            if (stringFields.contains(column)) {
                result = path;
                break;
            }
        }
        return result;
    }

    /**
     * 常用基础查询条件拼装
     *
     * @param jpaQuery   jpaQuery对象
     * @param qBaseModel qBaseModel
     * @param query      查询条件
     * @param <T>        范型对象
     */
    public static <T> void handleCommonJpaQuery(JPAQuery<T> jpaQuery, QBaseModel qBaseModel, TwQueryParam query) {
        jpaQuery.where(qBaseModel.deleteFlag.eq(0));
        if (!ObjectUtils.isEmpty(query.getId())) {
            jpaQuery.where(qBaseModel.id.eq(query.getId()));
        }
        if (!ObjectUtils.isEmpty(query.getCreateUserId())) {
            jpaQuery.where(qBaseModel.createUserId.eq(query.getCreateUserId()));
        }
        if (!ObjectUtils.isEmpty(query.getCreateTimeStart())) {
            jpaQuery.where(qBaseModel.createTime.goe(query.getCreateTimeStart()));
        }
        if (!ObjectUtils.isEmpty(query.getCreateTimeEnd())) {
            jpaQuery.where(qBaseModel.createTime.loe(query.getCreateTimeEnd()));
        }
        if (!ObjectUtils.isEmpty(query.getRemark())) {
            jpaQuery.where(qBaseModel.remark.like(SqlUtil.toSqlLikeString(query.getRemark())));
        }
    }

    /**
     * 必要的更新字段
     *
     * @param update     更新语句
     * @param qBaseModel 更新对象
     */
    public static void updateCommonJpaQuery(JPAUpdateClause update, QBaseModel qBaseModel) {
        SysUserDTO userDTO = GlobalUtil.getLoginUser();
        update.set(qBaseModel.modifyUserId, userDTO.getId());
        update.set(qBaseModel.updater, userDTO.getFirstName());
        update.set(qBaseModel.modifyTime, LocalDateTime.now());
    }
}
